#include <QGuiApplication>
#include <QQmlApplicationEngine>
#include <QObject>
#include <QQuickView>
#include <QQuickItem>
#include <QQmlContext>
#include <QMetaObject>
#include <QGuiApplication>

#include <memory>

#include "appmodel.h"

AppModel::AppModel()
{
    m_pScreen = QGuiApplication::primaryScreen();

    m_menu = std::make_shared<QMenu>();
    m_action = std::make_shared<QAction>(QStringLiteral("Launch flyout"));
    connect(m_action.get(), SIGNAL(triggered()), this, SLOT(onActionTriggered()));

    m_menu->addAction(m_action.get());

    m_icon = std::make_shared<QSystemTrayIcon>();
    m_icon->setToolTip(QStringLiteral("AnimationIssue test"));

    QPixmap pixmap(24, 24);
    pixmap.fill(QColor::fromRgb(255, 255, 0));

    connect(m_icon.get(), SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(onSysTrayActivated(QSystemTrayIcon::ActivationReason)));

    m_icon->setIcon(QIcon(pixmap));
    m_icon->setContextMenu(m_menu.get());
    m_icon->show();

}

void AppModel::onSysTrayActivated(QSystemTrayIcon::ActivationReason reason)
{
    if (reason == QSystemTrayIcon::Trigger)
    {
        launchFlyout();
    }
}
void AppModel::onActionTriggered()
{
    launchFlyout();
}

void AppModel::launchFlyout()
{
    const int flyoutWidth = 360;
    const int flyoutHeight = 600;

    if (m_pScreen)
    {
        if (m_view == nullptr)
        {
            m_view = std::make_shared<QQuickView>();

            m_view->setSource(QUrl(QStringLiteral("qrc:/Flyout.qml")));
            m_view->create();
        }

        m_view->hide();

        QPoint startPosition;
        QPoint finalPosition;

        QRect rectMonitor = m_pScreen->geometry();
        finalPosition.setX(rectMonitor.right() - rectMonitor.width()/2 - flyoutWidth/2);
        finalPosition.setY(rectMonitor.bottom() - flyoutHeight - 40);

        startPosition.setX(finalPosition.x());
        startPosition.setY(rectMonitor.bottom());

        m_view->setPosition(startPosition);
        m_view->show();

        QMetaObject::invokeMethod(m_view->rootObject(), "startPopupAnimation", Qt::DirectConnection, Q_ARG(QVariant, startPosition.y()), Q_ARG(QVariant, finalPosition.y()));
    }
}
